var Impact = (function () {
    'use strict';
    /**
     * @description Calculate the impact on the current node, in percent, by this preceding node (impactNode) and edge (criticality).
     * Its the default calculate edge impact used.
     * @param {number} impactNode Impact of the preceding node.
     * @param {number} criticality Criticality carry by the preceding edge.
     */
    function calculateImpactEdge (impactNode, criticality) {
        return impactNode * criticality / 100;
    }

    /**
     * @description Provide a helper to calculate the impact of a node on another one.
     * @typedef Impact
     * @constructor
     */
    function Impact () {
        /**
         * @description A collection of values who doesn't need specific treatment.
         * @type {Array<number>}
         * @private
         */
        var _normalValues = [];
        /**
         * @description An object who contains all the specific values.
         * Contains a property size used to know how much collection it contains.
         * WARNING: Additional treatment is used for percentage values.
         * @type {object}
         * @private
         */
        var _specificValues = { size : 0 };
        /**
         * @description Tell if the specific values belongs to redundant values or percentage values.
         * If true, specific values belongs to redundant values.
         * If false, specific values belongs to percentage values.
         * @type {boolean|undefined}
         * @private
         */
        var _redundant = undefined;
        /**
         * @description Set the '_redundant' property. This property can be set up once.
         * @param {boolean|string} redundant Boolean wanted, only "true" and "false" string authorized.
         */
        this.setRedundant = function (redundant) {
            if (typeof _redundant === "undefined")
                switch (typeof redundant) {
                    case "String":
                    case "string":
                        if (redundant === "true") {
                            _redundant = true;
                        } else if (redundant === "false") {
                            _redundant = false;
                        } else {
                            throw "Wrong string given. Authorized strings are : \"true\" or \"false\".";
                        }
                        break;
                    case "Boolean":
                    case "boolean":
                        _redundant = redundant;
                        break;
                    default:
                        _redundant = false;
                }
        };
        /**
         * @description Give the node impact.
         * @returns {number}
         */
        this.getImpactNodeValue = function () {
            if (_specificValues.size)
                /* S'il y a des éléments spécifique, ont récupère leur valeur que l'ont ajoute à '_normalValues' avant de retourné la valeur de celui-ci. */
                for (var index in _specificValues) {
                    if (index !== "size") {
                        var value;
                        if (_redundant) {
                            /* If the specific values belongs to redundant values. */
                            /* If there is less calculated values than 'nb_nodes', we add a value of 0 (we speculate that the others nodes impact not calculated is equal to the impact of a node who is fully operational). */
                            if (_specificValues[index].val.length !== _specificValues[index].nb_nodes) _specificValues[index].val.push(0);
                            /* We take the MIN impact of those values */
                            value = Math.min.apply(null, _specificValues[index].val);
                        } else {
                            /* If the specific values belongs to percentage values, we take the SUM of all these values.
                             * There initial value equal to 0 because it's the value of the impact of a fully operational node. */
                            value = _specificValues[index].val.reduce(function (precedent_value, current_value) {
                                return precedent_value + current_value;
                            }, 0);
                            value = (value > 100 ? 100 : value);
                            /* Fonction reduce de l'objet Array. Itère sur chaque element, donne l'élément précédent ainsi que l'élément courant.
                             * Si une valeur d'initialisation est passé en paramètre, ici 0, le premier élément sera cette valeur,
                             *    Sinon ce sera l'élément à la position 0 et l'élément courant sera celui à la position 1.
                             */
                        }
                        _normalValues.push(value);
                    }
                }

            /* We return the MAX impact of the '_normalValues' who contains the normal impact nodes and the specific nodes treated. */
            return Math.max.apply(null, _normalValues);
        };
        /**
         * @description Add a specific element in '_specificValues'.
         * @param {string} name The name of the specific element to add.
         * @param {number|string} nb_nodes Used only if the specific element belongs to a percentage element (authorized strings are number under string representation like "7").
         * @param {boolean} [redundant] Whether or not the specific element belongs to redundant values. If not they belongs to percentage values. Used to set up the '_redundant' property.
         */
        this.addSpecific = function (name, nb_nodes, redundant) {
            if (typeof _redundant !== "undefined") {
                _specificValues[name] = { nb_nodes : parseInt(nb_nodes) , val : [] };
                if (!_redundant) _specificValues[name].criticality = 100 / nb_nodes;
                _specificValues.size++;
            } else if (typeof redundant !== "undefined") {
                this.setRedundant(redundant);
                this.addSpecific(name, nb_nodes);
            }
        };
        /**
         * @description Add a specific value.
         * @param {string} name The name of the element to add a value.
         * @param {number|string} edge A preceding edge or the criticality of a preceding edge (authorized string are number under string representation like "100").
         * @param {number|string} node The source node of this 'edge' or the 'impactNode' data of the source node of this 'edge' (authorized string are number under string representation like "100").
         */
        this.addSpecificValue = function (name, edge, node) {
            _specificValues[name].val.push(Impact.prototype.performCalculation(edge, node));
        };
        /**
         * @description Add a normal value.
         * @param {number|string} edge A preceding edge or the criticality of a preceding edge (authorized string are number under string representation like "100").
         * @param {number|string} node The source node of this 'edge' or the 'impactNode' data of the source node of this 'edge' (authorized string are number under string representation like "100").
         */
        this.addNormalValue = function (edge, node) {
            _normalValues.push(Impact.prototype.performCalculation(edge, node));
        };
        /**
         * @description Test if specific elements with this 'name' exists.
         * @param {string} name The name of the specific element to test.
         * @returns {boolean}
         */
        this.specificExist = function (name) {
            return typeof _specificValues[name] !== "undefined";
        };
    }

    Impact.prototype = {
        /**
         * @description A function defined by the user to perform the impact edge calculation.
         * MUST RETURN A NUMBER !!!
         * @param {object} edge A preceding edge.
         * @param {object|number|string} node The source node of this 'edge' or the 'impactNode' data of the source node of this 'edge' (authorized string are number under string representation like "100").
         * @param {number|undefined} criticality The calculated criticality in case of this node belongs to a percentage node.
         * @returns {number} The impact calculate with this edge and this node.
         */
        calculateFunction : null,
        /**
         * @description Launch the appropriate calculation function.
         * @param {object} edge A preceding edge.
         * @param {object|number|string} node The source node of this 'edge' or the 'impactNode' data of the source node of this 'edge' (authorized string are number under string representation like "100").
         * @param {number} [criticality] The criticality to use during the impact calculation.
         */
        performCalculation : function (edge, node, criticality) {
            if (this.calculateFunction)    return this.calculateFunction(edge, node, criticality);
            /* Here we test the edge and node type to give the good value to the function. */
            else                           return calculateImpactEdge(node, edge);
        }
    };

    return Impact;
}());